#!/bin/sh
# the next line restarts using wish \
exec wish "$0" "$@"

#########################################################################
# winmmucl - Mark's MUd Client with a text widget to display output.
#
# Copyright (C) 1997 Mark Patton
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#########################################################################

# Edit the below paths or let make do it. 
# Do NOT edit anything else unless you know tcl/tk.
# Do NOT delete any of the preceding lines. Make depends on them.
set Mmucl(lib_dir) ..
set Mmucl(rc_dir) ..
###########################

# Make sure the paths are absolute
if {[file pathtype Mmucl(lib_dir)]=="relative"} {
    set Mmucl(lib_dir) [file join [pwd] $Mmucl(lib_dir)]
}
if {[file pathtype $Mmucl(rc_dir)]=="relative"} {
    set Mmucl(rc_dir) [file join [pwd] $Mmucl(rc_dir)]
}

# Make the package mechanism look in the right place
set auto_path [linsert $auto_path 0 $Mmucl(lib_dir)]

package require mmucl
package require mmuclGui
package require balloon

# Callbacks to display text.

proc echo {str {tag ""}} {
    .display configure -state normal
    .display insert end $str\n [split $tag \;]
    .display see end
    if {[.display index end]>500} {
	.display delete 0.0 50.0
    }
    .display configure -state disabled
}

# Ansi color support
# Split $str into a list. Each element of the list will be
# ansi codes then plain text.

proc display {str} {
    upvar \#0 Mmucl(ansi_color) ansi Mmucl(ansi_tags) tags

   .display configure -state normal
    regsub -all \r $str "" str
    
    if {$ansi} {
        foreach part [split \[m$str \033] {
	    regexp {\[([^m]*)m(.*)} $part x codes text
	    foreach code [split $codes \;] {
		if {$code==0} {
		    set tags {{} {} {}}
		} elseif {$code<8} {
		    set tags [lreplace $tags 0 0 $code]
		} elseif {$code<38} {
                    set tags [lreplace $tags 1 1 $code]
                } else {
                    set tags [lreplace $tags 2 2 $code]
                }
            }
	    .display insert end $text [join $tags]
	}
    } else {
        .display insert end $str
    }
    .display see end
    if {[.display index end]>500} {
        .display delete 0.0 50.0
    }
    .display configure -state disabled
}

# Doesn't do X fonts...
proc set_font {family size} {
    global Mmucl
 
    set normal [list $family $size normal]
    set bold [list $family $size bold]

    catch {
	.display configure -font $normal
	.input configure -font $normal
	.display tag configure 1 -font $bold
    }
    set font [.display cget -font]
    echo "** Font set to $font **"
    set Mmucl(font_family) [lindex $font 0]
    set Mmucl(font_size) [lindex $font 1]
}

source [file join $Mmucl(lib_dir) lib mmuclGui_init]

text .display -wrap word -yscrollcommand {.vscroll set}
scrollbar .vscroll -orient vertical -command {.display yview}

# Make a menu
frame .menu 
menubutton .menu.file -text File -menu .menu.file.m -underline 0
menu .menu.file.m -tearoff 0

.menu.file.m add command -label "Connect" -command dialog_connect
.menu.file.m add separator
.menu.file.m add command -label "Load file" \
    -command {tkOpen source -title {Load a File}}
.menu.file.m add command -label "Send file" \
    -command {tkOpen textin -title {Send a file to the mud}}
.menu.file.m add command -label "Save" \
    -command {tkSave save -title {Save Current Aliases and Actions}}
.menu.file.m add separator
.menu.file.m add command -label "Exit" -command \
    {tkDialog exit yes -type yesno -message "Really quit?"}

menubutton .menu.edit -text "Edit" -menu .menu.edit.m -underline 0
menu .menu.edit.m -tearoff 0
.menu.edit.m add command -label Copy -command {tk_textCopy .display}
.menu.edit.m add command -label Paste -command {tk_textPaste .input}
.menu.edit.m add separator
.menu.edit.m add command -label Aliases -command {dialog_alias}
.menu.edit.m add command -label Actions -command {dialog_action}
.menu.edit.m add command -label Characters -command {dialog_character}
.menu.edit.m add command -label {Key bindings} -command {dialog_bind}

menubutton .menu.option -text "Options" -menu .menu.option.m -underline 0
menu .menu.option.m -tearoff 0
.menu.option.m add checkbutton -label "Ansi color" -onvalue 1 \
    -variable Mmucl(ansi_color) -offvalue 0
.menu.option.m add separator

.menu.option.m add cascade -label "Reconnect" -menu .menu.option.m.recon
menu .menu.option.m.recon -tearoff 0
.menu.option.m.recon add radiobutton -label "Never" -value 0 \
    -variable Mmucl(reconnect)
.menu.option.m.recon add radiobutton -label "After timeout" -value 1 \
     -variable Mmucl(reconnect)
.menu.option.m.recon add radiobutton -label "Always" -value 2 \
    -variable Mmucl(reconnect)
.menu.option.m add separator

.menu.option.m add checkbutton -label "Local echo" -onvalue 1 -offvalue 0\
    -variable Mmucl(echo)
.menu.option.m add cascade -label "Echo color" -menu .menu.option.m.color
menu .menu.option.m.color -tearoff 0
foreach {c n} {black 30 red 31 green 32 yellow 33 blue 34 magenta 35 \
		   cyan 36 gray 37 white 0} {
.menu.option.m.color add radiobutton -label $c -value $n \
    -variable Mmucl(echo_color)
}
.menu.option.m add separator
.menu.option.m add checkbutton -label "Keep line" -onvalue 1 -offvalue 0\
    -variable Mmucl(keep)

if {[info commands font]!=""} {
.menu.option.m add separator
.menu.option.m add cascade -label "Font family" -menu .menu.option.m.font
menu .menu.option.m.font -tearoff 0 
foreach c [lsort [font families]] {
    .menu.option.m.font add radiobutton -label $c -value [string tolower $c] \
	-variable Mmucl(font_family) \
	-command {set_font $Mmucl(font_family) $Mmucl(font_size)}
}
.menu.option.m add cascade -label "Font size" -menu .menu.option.m.size
menu .menu.option.m.size -tearoff 0
foreach c {6 8 10 11 12 14 18} {
    .menu.option.m.size add radiobutton -label $c -value $c \
	-variable Mmucl(font_size) \
	    -command {set_font $Mmucl(font_family) $Mmucl(font_size)}
}
}
unset c n

menubutton .menu.help -text Help -menu .menu.help.m -underline 0
menu  .menu.help.m -tearoff 0
.menu.help.m add command -label About -command {help about}
.menu.help.m add command -label Contents -command help

pack .menu.file -side left
pack .menu.edit -side left
pack .menu.option -side left
pack .menu.help -side right

grid .menu -row 0 -column 0 -columnspan 2 -sticky ew
grid .tools -row 1 -column 0  -sticky ew
grid .display -row 2 -column 0 -sticky news
grid .vscroll -row 2 -column 1 -sticky ns
grid .input -row 3 -column 0 -columnspan 1 -sticky ew
grid columnconfigure . 0 -weight 1
grid rowconfigure . 2 -weight 1
wm title . Winmucl

# Ansi color support:
#.display tag configure 0 reset
#.display tag configure 1 highlight
.display tag configure 4  -underline 1
.display tag configure 5  -underline 1 ;# <-- Should flash.
#.display tag configure 7 reverse fg and bg?
.display tag configure 30 -foreground black
.display tag configure 31 -foreground red3
.display tag configure 32 -foreground green3
.display tag configure 33 -foreground yellow3
.display tag configure 34 -foreground blue2
.display tag configure 35 -foreground magenta3
.display tag configure 36 -foreground cyan3
.display tag configure 37 -foreground lightgray
# 40 is the text widget background
.display tag configure 41 -background red3
.display tag configure 42 -background green3
.display tag configure 43 -background yellow3
.display tag configure 44 -background blue2
.display tag configure 45 -background magenta3
.display tag configure 46 -background cyan3
.display tag configure 47 -background lightgray

# Make selection visible.
.display tag raise sel

# Bindings to scroll text.
bind .input <Next> {.display yview scroll 1 page}
bind .input <Prior> {.display yview scroll -1 page}

# Global vars...
set Mmucl(ansi_color) 1
set Mmucl(ansi_tags) {"" 37 ""}
set Mmucl(font_family) [string tolower [lindex [.display cget -font] 0]]
set Mmucl(font_size) [lindex  [.display cget -font] 1]

# Use provided html browser as default.
set Mmucl(browser) [list [info nameofexecutable] [file join [file dirname [info script]] html_browse.tcl] -- %1]

# Call set_font to initialize some tags
set_font $Mmucl(font_family) $Mmucl(font_size)

echo "Mmucl version 1.0b4\n"