#!/bin/sh
# the next line restarts using wish \
exec wish "$0" "$@"

#########################################################################
# xmmucl - Mark's MUd Client using a console for output.
#
# Copyright (C) 1997 Mark Patton
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#########################################################################

# Edit the below paths or let make do it.
# Do NOT edit anything else unless you know tcl/tk.
# Do NOT delete any of the preceding lines. Make depends on them.
set Mmucl(lib_dir) ..
set Mmucl(rc_dir) ..
############################

# Make sure the paths are absolute
if {[file pathtype Mmucl(lib_dir)]=="relative"} {
    set Mmucl(lib_dir) [file join [pwd] $Mmucl(lib_dir)]
}
if {[file pathtype $Mmucl(rc_dir)]=="relative"} {
    set Mmucl(rc_dir) [file join [pwd] $Mmucl(rc_dir)]
}

# Make the package mechanism look in the right place
set auto_path [linsert $auto_path 0 $Mmucl(lib_dir)]

package require mmucl
package require mmuclGui
package require balloon

# Mmucl callbacks for display
proc display {str} {    
    puts -nonewline stdout $str
    flush stdout
}

proc echo {str {ansi_code ""}} {
    if {$ansi_code!=""} {set str "\033\[${ansi_code}m${str}\033\[0\;37\;40m"}
    puts stdout $str
    flush stdout
}

source [file join $Mmucl(lib_dir) lib mmuclGui_init]

# Make an options menu button in the toolbar.
menubutton .tools.option -text "Options" -menu .tools.option.m -bd 2 \
    -relief raised
menu .tools.option.m -tearoff 0
.tools.option.m add cascade -label "Reconnect" -menu .tools.option.m.recon
menu .tools.option.m.recon -tearoff 0
.tools.option.m.recon add radiobutton -label "Never" -value 0 \
    -variable Mmucl(reconnect)
.tools.option.m.recon add radiobutton -label "After timeout" -value 1 \
     -variable Mmucl(reconnect)
.tools.option.m.recon add radiobutton -label "Always" -value 2 \
    -variable Mmucl(reconnect)
.tools.option.m add separator

.tools.option.m add checkbutton -label "Local echo" -onvalue 1 -offvalue 0\
    -variable Mmucl(echo)
.tools.option.m add cascade -label "Echo color" -menu .tools.option.m.color
menu .tools.option.m.color -tearoff 0
foreach {c n} {black 30 red 31 green 32 yellow 33 blue 34 magenta 35 \
		   cyan 36 gray 37 white 0} {
.tools.option.m.color add radiobutton -label $c -value $n \
    -variable Mmucl(echo_color)
}
unset c n

.tools.option.m add separator
.tools.option.m add checkbutton -label "Keep line" -onvalue 1 -offvalue 0\
    -variable Mmucl(keep)
pack .tools.option -side left -padx 2
balloon_add .tools.option "User preferences"

.input configure -width 55

pack .tools -side top -fill x
pack .input -side top -fill x
wm resizable . 1 0
wm title . Xmmucl

# Accept input from the console.
fconfigure stdin -blocking 0 -buffering line
fileevent stdin readable {mmucl_parse [gets stdin]}

# Use provided html browser as default.
set Mmucl(browser) [list [info nameofexecutable] [file join [file dirname [info script]] html_browse.tcl] -- %1]

echo "Mmucl version 1.0b4\n"
